from django.db import models
from armstrong.core.arm_content.mixins.images import BaseThumbnailMixin
from ...fields import (RelatedContentField, RelatedObjectsField,
        ReverseRelatedObjectsField)


class Article(models.Model):
    title = models.CharField(max_length=250)
    related_content = RelatedContentField()
    related = RelatedObjectsField()
    reverse_related = ReverseRelatedObjectsField()


class TestingThumbnailMixin(BaseThumbnailMixin):
    def render_visual(self, preset_label, presets=None, defaults=None,
            *args, **kwargs):
        return "Render: %s" % preset_label

    def get_visual_thumbnail_url(self, preset_label, presets=None,
            defaults=None, *args, **kwargs):
        return "Thumb: %s" % preset_label


class Image(models.Model, TestingThumbnailMixin):
    title = models.CharField(max_length=250)



from django.contrib.contenttypes.generic import GenericRelation
from django.contrib.contenttypes.models import ContentType
from django.db.models.query import QuerySet

from armstrong.core.arm_wells.querysets import GenericForeignKeyQuerySet

from .models import RelatedContent


class BoundRelatedContentAccessor(object):
    def __init__(self, instance, model, from_field, to_field, *args, **kwargs):
        self.instance = instance
        self.model = model
        self.from_field = from_field
        self.to_field = to_field
        self.params = None

    def get_query_set_params(self):
        if self.params:
            return self.params
        for field in self.model._meta.virtual_fields:
            if field.name == self.from_field:
                self.params = {
                    field.ct_field: ContentType.objects.get_for_model(
                            self.instance).pk,
                    field.fk_field: self.instance.pk
                }
                break
        else:
            self.params = {self.from_field: self.instance}
        return self.params

    def get_query_set(self):
        qs = QuerySet(self.model)
        return qs.filter(**self.get_query_set_params())

    def all(self):
        qs = self.get_query_set()
        return GenericForeignKeyQuerySet(qs, gfk=self.to_field)

    def by_type(self, type):
        qs = self.get_query_set().filter(related_type__title=type)
        return GenericForeignKeyQuerySet(qs, gfk=self.to_field)

    def __getitem__(self, index):
        """
        See ``RelatedContentQuerySet.__getitem__``
        """
        if isinstance(index, basestring):
            return self.by_type(index)
        else:
            raise IndexError("RelatedFields can only be filtered by "
                    "related_type title")


class NonAssignableError(RuntimeError):
    pass


class RelatedM2MDescriptor(object):
    def __init__(self, model=RelatedContent, from_field="source_object",
            to_field="destination_object"):
        self.model = model
        self.from_field = from_field
        self.to_field = to_field

    def __get__(self, instance, cls=None):
        return BoundRelatedContentAccessor(
            instance=instance,
            model=self.model,
            from_field=self.from_field,
            to_field=self.to_field,
        )

    def __set__(self, instance, value):
        raise NonAssignableError("RelatedFields cannot be used to assign "
                "RelatedContent objects, use a RelatedContentField")


class RelatedObjectsField(object):
    def __init__(self, model=RelatedContent, from_field="source_object",
            to_field="destination_object"):
        self.model = model
        self.from_field = from_field
        self.to_field = to_field

    def contribute_to_class(self, cls, name):
        setattr(cls, name, RelatedM2MDescriptor(
                        model=self.model,
                        from_field=self.from_field,
                        to_field=self.to_field
                    )
        )


class ReverseRelatedObjectsField(RelatedObjectsField):
    def __init__(self, model=RelatedContent, from_field="destination_object",
            to_field="source_object"):
        super(ReverseRelatedObjectsField, self).__init__(
                model, from_field, to_field)


class RelatedContentField(GenericRelation):
    def __init__(self, **kwargs):
        defaults = {
            "object_id_field": "source_id",
            "content_type_field": "source_type",
        }
        defaults.update(kwargs)
        super(RelatedContentField, self).__init__(RelatedContent, **defaults)


from south.modelsinspector import add_ignored_fields
add_ignored_fields(
        ["^armstrong\.apps\.related_content\.fields\.RelatedContentField"])



from django import forms
from django.conf import settings
from django.contrib import admin
from django.contrib.contenttypes.generic import GenericTabularInline

from armstrong.hatband import widgets

from .models import RelatedContent
from .models import RelatedType


"""
Setting ARMSTRONG_RELATED_TYPE_DEFAULT_FILTER allows you to specify
values to provide to ``get`` that looks up the initial value for
"""
RELATED_TYPE_INITIAL_FILTER = "ARMSTRONG_RELATED_TYPE_INITIAL_FILTER"


class RelatedContentInlineForm(forms.ModelForm):
    class Meta:
        widgets = {
            "destination_type": forms.HiddenInput(),
            "destination_id": widgets.GenericKeyWidget(
                object_id_name="destination_id",
                content_type_name="destination_type",
            ),
            "order": forms.HiddenInput(),
        }


class RelatedContentInline(GenericTabularInline):
    ct_field = "source_type"
    ct_fk_field = "source_id"

    model = RelatedContent
    template = "admin/edit_inline/generickey.html"

    form = RelatedContentInlineForm

    def formfield_for_foreignkey(self, *args, **kwargs):
        args, kwargs = formfield_for_foreignkey_helper(self, *args, **kwargs)
        return super(RelatedContentInline, self).formfield_for_foreignkey(
                *args, **kwargs)


def formfield_for_foreignkey_helper(inline, *args, **kwargs):
    """
    The implementation for ``RelatedContentInline.formfield_for_foreignkey``

    This takes the takes all of the ``args`` and ``kwargs`` from the call to
    ``formfield_for_foreignkey`` and operates on this.  It returns the updated
    ``args`` and ``kwargs`` to be passed on to ``super``.

    This is solely an implementation detail as it's easier to test a function
    than to provide all of the expectations that the ``GenericTabularInline``
    has.
    """
    db_field = args[0]
    if db_field.name != "related_type":
        return args, kwargs

    initial_filter = getattr(settings, RELATED_TYPE_INITIAL_FILTER,
            False)
    if "initial" not in kwargs and initial_filter:
        # TODO: handle gracefully if unable to load and in non-debug
        initial = RelatedType.objects.get(**initial_filter).pk
        kwargs["initial"] = initial
    return args, kwargs


admin.site.register(RelatedType)
